@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2017 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2019 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -2205,7 +2205,8 @@ CMD(debug)
    }
 
    if(debug_file_name && trunc)
-      truncate(debug_file_name,0);
+      if(truncate(debug_file_name,0) < 0)
+         fprintf(stderr, "truncate failed: %s\n", strerror(errno));
 
    const char *c="debug";
    ResMgr::Set("log:file",c,debug_file_name?debug_file_name:"");
@@ -2800,7 +2801,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2017);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2019);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -2833,7 +2834,7 @@ CMD(ver)
    {
       const char *lib_name;
       const char *symbol;
-      enum type_t { STRING_PTR, FUNC0, INT8_8 } type;
+      enum type_t { STRING_OR_PTR, STRING_PTR, FUNC0, INT8_8 } type;
       const char *skip_prefix;
       typedef const char *(*func0)(void *);
       const char *query() const
@@ -2845,6 +2846,11 @@ CMD(ver)
 	    const char *str=0;
 	    switch(type)
 	    {
+	    case STRING_OR_PTR:
+	       str=(const char*)sym_ptr;
+	       if(skip_prefix && !strncmp(str,skip_prefix,sizeof(char*)))
+		  break;
+	       // FALLTHROUGH
 	    case STRING_PTR:
 	       str=*(const char**)sym_ptr;
 	       break;
@@ -2868,7 +2874,7 @@ CMD(ver)
       {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
       {"idn2",	     "idn2_check_version",   VersionInfo::FUNC0,     0},
       {"libiconv",   "_libiconv_version",    VersionInfo::INT8_8,    0},
-      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
+      {"OpenSSL",    "SSL_version_str",      VersionInfo::STRING_OR_PTR,"OpenSSL "},
       {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
       {"zlib",	     "zlibVersion",	     VersionInfo::FUNC0,     0},
       {0}
@@ -3004,12 +3010,13 @@ CMD(bookmark)
    {
       lftp_bookmarks.Remove(""); // force bookmark file creation
 
-      xstring cmd0("exec ${EDITOR:-vi} ");
-      cmd0.append(shell_encode(lftp_bookmarks.GetFilePath()));
-      xstring cmd1("/bin/sh -c ");
-      cmd1.append(shell_encode(cmd0));
-
-      parent->PrependCmd(xstring::get_tmp("shell ").append_quoted(cmd1));
+      const char *bin=getenv("EDITOR");
+      if (bin==NULL)
+         bin="vi";
+      xstring cmd(bin);
+      cmd.append(" ");
+      cmd.append(shell_encode(lftp_bookmarks.GetFilePath()));
+      parent->PrependCmd(xstring::get_tmp("shell ").append_quoted(cmd));
    }
    else if(!strcasecmp(op,"import"))
    {
