@@ -61,7 +61,6 @@ CDECL_END
 #include "url.h"
 #include "ResMgr.h"
 #include <mbswidth.h>
-#include "strftime.h"
 
 const char *dir_file(const char *dir,const char *file)
 {
@@ -958,31 +957,44 @@ const xstring& shell_encode(const char *string,int len)
    return (result);
 }
 
-void remove_tags(char *buf)
+int remove_tags(char *buf)
 {
    int len=strlen(buf);
-   for(;;)
+   int less = -1;
+   for(int i = 0; i < len;i++)
    {
-      char *less=strchr(buf,'<');
-      char *amp=strstr(buf,"&nbsp;");
-      if(!less && !amp)
-	 break;
-      if(amp && (!less || amp<less))
-      {
-	 amp[0]=' ';
-	 memmove(amp+1,amp+6,len-(amp+6-buf)+1);
-	 len-=amp+6-buf;
-	 buf=amp+1;
-	 continue;
+      if(strcmp(buf + i, "&nbsp;") == 0){
+        for(int j = 0; j < 6; j++)buf[i + j] = 0;
+        buf[i] = ' ';
+        i += 6;
+        i--;
+        continue;
+      }
+      if(buf[i] == '<'){
+        less = i;
+        continue;
+
+      }
+      if(buf[i] == '>'){
+        if(less != -1){
+          for(int j = less; j <= i; j++)buf[j] = 0;
+          less = -1;
+        }
+
       }
-      char *more=strchr(less+1,'>');
-      if(!more)
-	 break;
-      memmove(less,more+1,len-(more+1-buf)+1);
-      len-=more+1-buf;
-      buf=less;
    }
+   int zero = 0;
+   for(int i = 0; i < len;i++)
+   {
+     while(zero < i && buf[zero] != 0)zero++;
+     if(buf[i] != 0 and zero != i){
+      buf[zero] = buf[i];
+      buf[i] = 0;
+     }
+   }
+   return ++zero;
 }
+
 void rtrim(char *s)
 {
    int len=strlen(s);
