@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2019 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: network.cc,v 1.11 2011/05/10 05:53:20 lav Exp $ */
-
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -48,15 +46,10 @@
 #include "ProtoLog.h"
 #include "xstring.h"
 
-sockaddr_u::sockaddr_u()
-{
-   memset(this,0,sizeof(*this));
-}
-
 const char *sockaddr_u::address() const
 {
 #ifdef HAVE_GETNAMEINFO
-   char *buf=xstring::tmp_buf(NI_MAXHOST);
+   static char buf[NI_MAXHOST];
    if(getnameinfo(&sa,addr_len(),buf,NI_MAXHOST,0,0,NI_NUMERICHOST)<0)
       return "????";
    return buf;
@@ -79,8 +72,17 @@ int sockaddr_u::port() const
 #endif
    return 0;
 }
+void sockaddr_u::set_port(int port)
+{
+   if(sa.sa_family==AF_INET)
+      in.sin_port=htons(port);
+#if INET6
+   if(sa.sa_family==AF_INET6)
+      in6.sin6_port=htons(port);
+#endif
+}
 
-const xstring& sockaddr_u::to_string() const
+const xstring& sockaddr_u::to_xstring() const
 {
    return xstring::format("[%s]:%d",address(),port());
 }
@@ -94,6 +96,13 @@ bool sockaddr_u::is_reserved() const
 	  || (a[0]==127 && !is_loopback())
 	  || (a[0]>=240);
    }
+#if INET6
+   if(family()==AF_INET6) {
+      return IN6_IS_ADDR_UNSPECIFIED(&in6.sin6_addr)
+	  || IN6_IS_ADDR_V4MAPPED(&in6.sin6_addr)
+	  || IN6_IS_ADDR_V4COMPAT(&in6.sin6_addr);
+   }
+#endif
    return false;
 }
 
@@ -104,6 +113,10 @@ bool sockaddr_u::is_multicast() const
       unsigned char *a=(unsigned char *)&in.sin_addr;
       return (a[0]>=224 && a[0]<240);
    }
+#if INET6
+   if(family()==AF_INET6)
+      return IN6_IS_ADDR_MULTICAST(&in6.sin6_addr);
+#endif
    return false;
 }
 
@@ -114,8 +127,15 @@ bool sockaddr_u::is_private() const
       unsigned char *a=(unsigned char *)&in.sin_addr;
       return (a[0]==10)
 	  || (a[0]==172 && a[1]>=16 && a[1]<32)
-	  || (a[0]==192 && a[1]==168);
+	  || (a[0]==192 && a[1]==168)
+	  || (a[0]==169 && a[1]==254); // self-assigned
    }
+#if INET6
+   if(family()==AF_INET6) {
+      return IN6_IS_ADDR_SITELOCAL(&in6.sin6_addr)
+	  || IN6_IS_ADDR_LINKLOCAL(&in6.sin6_addr);
+   }
+#endif
    return false;
 }
 bool sockaddr_u::is_loopback() const
@@ -131,6 +151,64 @@ bool sockaddr_u::is_loopback() const
 #endif
    return false;
 }
+bool sockaddr_u::is_compatible(const sockaddr_u& o) const
+{
+   return family()==o.family()
+      && !is_multicast() && !o.is_multicast()
+      && !is_reserved() && !o.is_reserved()
+      && is_private()==o.is_private()
+      && is_loopback()==o.is_loopback();
+}
+
+bool sockaddr_u::set_compact(const char *c,size_t len)
+{
+   if(len==4) {
+      sa.sa_family=AF_INET;
+      memcpy(&in.sin_addr,c,4);
+      in.sin_port=0;
+      return true;
+#if INET6
+   } else if(len==16) {
+      sa.sa_family=AF_INET6;
+      memcpy(&in6.sin6_addr,c,16);
+      return true;
+#endif
+   } else if(len==6) {
+      sa.sa_family=AF_INET;
+      memcpy(&in.sin_addr,c,4);
+      in.sin_port=htons((c[5]&255)|((c[4]&255)<<8));
+      return true;
+#if INET6
+   } else if(len==18) {
+      sa.sa_family=AF_INET6;
+      memcpy(&in6.sin6_addr,c,16);
+      in6.sin6_port=htons((c[17]&255)|((c[16]&255)<<8));
+      return true;
+#endif
+   }
+   return false;
+}
+const sockaddr_compact& sockaddr_u::compact() const
+{
+   sockaddr_compact& c=compact_addr();
+   int p=port();
+   if(c.length() && p) {
+      c.append(char(p>>8));
+      c.append(char(p&255));
+   }
+   return c;
+}
+sockaddr_compact& sockaddr_u::compact_addr() const
+{
+   sockaddr_compact& c=sockaddr_compact::get_tmp();
+   if(family()==AF_INET)
+      c.append((const char*)&in.sin_addr,4);
+#if INET6
+   else if(family()==AF_INET6)
+      c.append((const char*)&in6.sin6_addr,16);
+#endif
+   return c;
+}
 
 void Networker::NonBlock(int fd)
 {
@@ -227,7 +305,7 @@ void Networker::SocketBindStd(int s,int af,const char *hostname,int port)
    if(bind_addr.set_defaults(af,hostname,port))
    {
       if(bind_addr.bind_to(s)==-1)
-	 ProtoLog::LogError(0,"bind(%s): %s",bind_addr.to_string().get(),strerror(errno));
+	 ProtoLog::LogError(0,"bind(%s): %s",bind_addr.to_string(),strerror(errno));
    }
 }
 int Networker::SocketCreate(int af,int type,int proto,const char *hostname)
@@ -284,7 +362,7 @@ int Networker::SocketAccept(int fd,sockaddr_u *u,const char *hostname)
 
 void Networker::SocketSinglePF(int s,int pf)
 {
-#if INET6
+#if INET6 && defined(IPV6_V6ONLY)
    if(pf==PF_INET6) {
       int on = 1;
       if(-1==setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&on, sizeof(on)))
@@ -349,3 +427,61 @@ int Networker::SocketBuffered(int sock)
    return 0;
 #endif
 }
+
+#ifdef HAVE_IFADDRS_H
+#include <ifaddrs.h>
+#endif
+const char *Networker::FindGlobalIPv6Address()
+{
+#if INET6 && defined(HAVE_IFADDRS_H)
+   struct ifaddrs *ifaddrs=0;
+   getifaddrs(&ifaddrs);
+   for(struct ifaddrs *ifa=ifaddrs; ifa; ifa=ifa->ifa_next) {
+      if(ifa->ifa_addr && ifa->ifa_addr->sa_family==AF_INET6) {
+	 struct in6_addr *addr=&((struct sockaddr_in6*)ifa->ifa_addr)->sin6_addr;
+	 if(!IN6_IS_ADDR_UNSPECIFIED(addr) && !IN6_IS_ADDR_LOOPBACK(addr)
+	 && !IN6_IS_ADDR_LINKLOCAL(addr) && !IN6_IS_ADDR_SITELOCAL(addr)
+	 && !IN6_IS_ADDR_MULTICAST(addr)) {
+	    char *buf=xstring::tmp_buf(INET6_ADDRSTRLEN);
+	    inet_ntop(AF_INET6, addr, buf, INET6_ADDRSTRLEN);
+	    freeifaddrs(ifaddrs);
+	    return buf;
+	 }
+      }
+   }
+   freeifaddrs(ifaddrs);
+#endif
+   return 0;
+}
+
+static bool CanCreateIpv6Socket()
+{
+#if INET6
+   bool can=true;
+   int s=socket(AF_INET6,SOCK_STREAM,IPPROTO_TCP);
+   if(s==-1 && (errno==EINVAL
+#ifdef EAFNOSUPPORT
+      || errno==EAFNOSUPPORT
+#endif
+   ))
+      can=false;
+   if(s!=-1)
+      close(s);
+   return can;
+#else
+   return false;
+#endif
+}
+
+static struct NetworkInit : private Networker {
+   NetworkInit();
+} NETWORK_INIT;
+
+NetworkInit::NetworkInit()
+{
+#if INET6
+   // check if ipv6 is really supported
+   if(!Networker::FindGlobalIPv6Address() || !CanCreateIpv6Socket())
+      ResMgr::Set("dns:order",0,"inet");
+#endif // INET6
+}
